% Inputs:
% T is the (finite) time horizon
% Pdead is an array, where Pdead(tm) = Prob{D <= t), where tm=t+1.
% c>0 is the cost of continuing, d>0 is the cost of going past the deadline.
% To get reasonable behavior, one should have c<1<d, since 1 is the cost of
% guessing the hypothesis incorrectly.
%
% Returns the value function, the q-factors, and the upper and lower bounds
% of the windows.
% 
% We assume that our observations are Bernoulli random variables under both
% hypotheses.  q1 is the probability of seeing a 1 under H_1, and q0 is the
% probability of seeing a 1 under H_0.
% 
function [V,Qgo,Qstop,window_lb,window_ub] = TabularTimingUncertainty(T,Pdead,c,d,q0,q1,nchunks)
	LikelihoodRatio0=(1-q1)/(1-q0); % Likelihood Ratio H1/H0 if we observe 0
	LikelihoodRatio1=q1/q0; % Likelihood Ratio H1/H0 if we observe 1

	% These are the chunked, and unchunked version of Prob{\theta=1 | \Ft}.
	pchunk=[1:nchunks];
	p = fromchunk(pchunk,nchunks);

	% prob0 and prob1 are the probabilities of observing 0
	% and 1, respectively at time t+1, given that p^t=p.
	prob0 = (1-q0)*(1-p) + (1-q1)*p; 
	prob1 = q0*(1-p) + q1*p;

	% Compute the odds ratio now (OddsRatio), and what the
	% odds ratio will be if we observe a 0 (OddsRatio0) or
	% a 1 (OddsRatio1).  Note that these odds ratios are allowed to
	% be infinite.
	phi = OddsRatio(p);
	phi0 = phi * LikelihoodRatio0;
	phi1 = phi * LikelihoodRatio1;

	% p0 (p1) are the values of p^{t+1} if we observe a 0
	% (a 1, respectively), given that p^t=p.  To get these,
	% we apply the functional inverse of p/(1-p) to the
	% respective odds ratios.  This function inverse handles values
	% between 0 and Inf, inclusive.
	p0 = InvertOddsRatio(phi0);
	p1 = InvertOddsRatio(phi1);

	% Our indexing is a little funny.  Qgo(t) and Qstop(t) are actually the
	% Q-factors for going and stopping at time t-1.  Value(t) is the value
	% function for time t-1.
	t=T; Tm=T+1; % T+1 is matlab indexing for real time T.
	Qgo(Tm,pchunk) = inf;
	Qstop(Tm,pchunk) = min(p,1-p) + c*T;
	V(Tm,pchunk) = Qstop(Tm,pchunk);
	window_lb(Tm) = .5; % The continue set is empty at the terminal time. 
        window_ub(Tm) = .5;

	for t=[T-1:-1:0]
		tm = t+1; % tm is matlab indexing, t is real indexing

		% Qgo(t,p) is the expectation of V(t+1,p^{t+1}) given p^t=p.
		% Our observation at time t+1 is either a 1 or a 0, so this
		% expectation is the sum of two terms, one for each of these
		% possible observations.

		% V0 and V1 are the values of the value function at
		% time t+1 given that we observe a 0 or 1,
		% respectively.
		V0 = V(tm+1,tochunk(p0,nchunks));
		V1 = V(tm+1,tochunk(p1,nchunks));

		if (Pdead(tm) < 1) 
			Pdie = (Pdead(tm+1)-Pdead(tm))/(1-Pdead(tm));
		else
			Pdie = 1; % Conditional probability given a null event is undefined.
		end
		Qgo(tm,pchunk) = Pdie*(d+c*(t+1)) + (1-Pdie)*(prob0.*V0 + prob1.*V1);

		% Qstop(tm) is the cost of stopping at time t.  If we
		% stop before the deadline, it is the error
		% probability.  If we stop after the deadline, it is
		% the penalty d.
		Qstop(tm,pchunk) = min(p,1-p) + c*t;

		% V(tm) is the value function at time t.
		V(tm,pchunk) = min(Qstop(tm,pchunk),Qgo(tm,pchunk));

		% We calculate the continuation region.  The continuation
		% region is composed of all points such that Qgo(tm,:) <
		% Qstop(tm,:).
		region = find(Qgo(tm,:)<Qstop(tm,:));
		if (isempty(region))
			window_lb(tm)=.5;
			window_ub(tm)=.5;
		else
			window_lb(tm) = fromchunk(min(region),nchunks);
			window_ub(tm) = fromchunk(max(region),nchunks);
		end
	end
end
