% Simulates a narrowing window sequential probability ratio test and 
% returns a list of the response times observed, and whether the guess was
% correct.  It assumes that the deadline is completely unobserved, and does not
% report timeouts.
% The window_lb variable should contain a vector of lower bounds on the
% continuation window, and window_ub should contain a vector of upper
% bounds.  Note that real time t is indexed by t+1.  So if p^t <=
% window_lb(t+1), then the test will stop at (real, not matlab) time t.
% Setting window_lb(t+1)=0 and window_ub(t+1)=1 means it will never stop
% (since p^t>0 almost surely) at time t.
% Setting window_ub(t+1)=.5 and window_lb(t+1)=.5 means it will always stop
% at time t (given that it doesn't stop earlier).  We assume that if a
% sample continues until the window array has no more elements, then we
% stop right afterward.  This is the same as assuming an window entry of .5
% after the end.
%
% Returns response times and whether it was correct for each trial,
% and also estimates of the probability that the test guesses correctly,
% given that it stops at time t.  These are returned as an MLE pmle(t), and
% a confidence region [plb(t),pub(t)] (here lb is lower bound and ub is
% upper bound).  Also, it returns num(t), which is the number of people
% stopping at time t, and numcorrect(t), which is the number of people
% stopping and guessing correctly at time t.
function [num_total, num_correct, pmle, plb, pub] = SimulateResponseTimes(window_lb, window_ub, q0, q1, nsamples)
	T = length(window_lb)-1;
    phi_ub = OddsRatio(window_ub); % Convert probabilities to odds ratios.
    phi_lb = OddsRatio(window_lb);
    LikelihoodRatio1 = q1/q0;
    LikelihoodRatio0 = (1-q1)/(1-q0);
    % num_total(t+1) is the number of samples that had response time t.
    % num_correct(t+1) is the number of these that had correct guesses.
    num_total = zeros(1,T+1);
    num_correct = zeros(1,T+1);
	for n=1:nsamples
        phi = 1; % Corresponds to p=1/2, as hardcoded on the next line.
		theta = rand < 1/2; % Theta is a Bernoulli random variable.
		if (theta==1)
            probObserve1 = q1; % Probability of observing a 1.
		else % theta==0
            probObserve1 = q0;
        end
        % I could implement this next block via matrix manipulation rather
        % than a loop, by generating T+1 random Bernoulli trials
        % beforehand, but would it save time?  An initial test indicated
        % not.  On one hand, Matlab is slow at looping, but on the other
        % hand, the looping implementation gets to exit early when exits
        % the window, which is often much before T.  A third alternative
        % would be to generate the random variables in chunks -- generate
        % 10 at a time, and keep the loop.  That code would be even more
        % complex though.
		for t=0:T
			tm = t+1;
            % Stop if we exit the window
        	if (phi <= phi_lb(tm) || phi>=phi_ub(tm))
				break;
            end

			% Our observation is a 0 or 1, and the probability
			% distribution depends upon theta. We compute the likelihood
			% ratio between the two hypotheses based upon the observation
			% above in the initial part of the function, then multiply it
			% by the old odds ratio to get the new odds ratio.
            x = rand < probObserve1;
			if (x==1)
				phi = LikelihoodRatio1 * phi;
			else % x==0
             	phi = LikelihoodRatio0 * phi;
            end
        end
        tm = t+1;
		num_total(tm) = num_total(tm) + 1;
        guess = phi > 1; % We guess H_1 if p^\tau > 1/2.
        if (guess==theta)
            num_correct(tm) = num_correct(tm) + 1;
        end
    end
    
    % Estimate the probability that the SPRT with the given window guesses
    % the correct hypothesis, given that it stops at time t.
    for t=[0:T]
        tm=t+1;
        if (num_total(tm)>0)
            pmle(tm) = num_correct(tm)/num_total(tm);
        else
            pmle(tm) = NaN;
        end
        %[plb(tm),pub(tm)] = FrequentistBernoulliConfidenceInterval(num_correct(tm),num_total(tm));
        [plb(tm),pub(tm)] = BayesBernoulliConfidenceInterval(num_correct(tm),num_total(tm));
    end
end