% Inputs:
% T is the (finite) time horizon
% Pdead is an array, where Pdead(tm) = Prob{D <= t), where tm=t+1.
% c>0 is the cost of continuing, d>0 is the cost of going past the deadline.
% To get reasonable behavior, one should have c<1<d, since 1 is the cost of
% guessing the hypothesis incorrectly.
% 
% We assume that our observations are Bernoulli random variables under both
% hypotheses.  q is the probability of seeing a 1 under H_1, and 1-q is the
% probability of seeing a 0 under H_0.
% 
function [V,Qgo,Qstop,window] = ComputeQgo(T,Pdead,c,d,q)
	syms p x ... real

	% The following block of computations are stationary in time, so we
	% need only do them once.
	OddsRatio = p/(1-p);
	OddsRatioInv = x/(1+x); % Inverse of OddsRatio
	LikelihoodRatio0 = (1-q)/q; % Likelihood Ratio if we observe 0
	LikelihoodRatio1 = (1-q)/q; % Likelihood Ratio if we observe 1
	% nextp0 (nextp1) are the values of p^{t+1} if we observe a 0
	% (a 1, respectively), given that p^t=p.
	nextp0 = compose(OddsRatioInv, OddsRatio*LikelihoodRatio0);
	nextp1 = compose(OddsRatioInv, OddsRatio*LikelihoodRatio1);
	% prob0 and prob1 are the probabilities of observing 0 and 1,
	% respectively at time t+1, given that p^t=p.
	prob0 = q*(1-p) + (1-q)*p; 
	prob1 = q*p + (1-q)*(1-p);

	% Our indexing is a little funny.  Qgo(t) and Qstop(t) are actually the
	% Q-factors for going and stopping at time t-1.  Value(t) is the value
	% function for time t-1.
	t=T, Tm=T+1; % T+1 is matlab indexing for real time T.
	Qgo(Tm) = inf;
	Qstop(Tm) = min(p,1-p) * (1-Pdead(Tm)) + c*T + d*Pdead(Tm);
	V(Tm) = Qstop(Tm);
	window(Tm) = .5; % The continue set is empty at the terminal time. 
	for t=[T-1:-1:0]
		tm = t+1; % tm is matlab indexing, t is real indexing
		% Qgo(t,p) is the expectation of V(t+1,p^{t+1}) given p^t=p.
		% Our observation at time t+1 is either a 1 or a 0, so this
		% expectation is the sum of two terms, one for each of these
		% possible observations.
		V0 = compose(V(tm+1), nextp0);
		V1 = compose(V(tm+1), nextp1)
		Qgo(tm) = prob0*V0 + prob1*V1;

		% Qstop(tm) is the value of stopping at time t.
		Qstop(tm) = min(p,1-p)*(1-Pdead(tm)) + c*t + d*Pdead(tm);
		% V(tm) is the value function at time t.
		V(tm) = min(Qstop(tm),Qgo(tm));

		% We calculate the lower bound of the continuation
		% region.  If the continuation region is empty, this is 1/2,
		% and otherwise it is the smallest point such that Qgo(tm) >=
		% Qstop(tm).  At this point, the inequality is actually
		% equality.  It will always be in the interval (0,1/2].
		window(tm) = solve(Qgo(tm)-Qstop(tm));
	end
end
